/*======================================================================*/
/* odpok v0.3 by Nelo (nelo@message.sk) (msdos version)                 */
/* simple skin-resistance bio-feedback meter                            */
/* (reading PIN#3 on the game port)                                     */
/*======================================================================*/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include <dos.h>
#include <conio.h>
#include <time.h>

#define GADDR 0x201     /* gameport address */
#define MAXRES 100000   /* maximum measurable resistance */
#define MAXCALM 400     /* maximum calmness deviation in one step */
#define CAPACLR 20000   /* delay to clear the capacitors */

#define DELAY 2         /* delay between readings (sec) */
#define ITER 10         /* #of iterations */

typedef enum { FALSE = 0, TRUE = 1} BOOL; /* BOOL type */
int d_delay = DELAY;
int d_iter = ITER;
int gaddr = GADDR;  /* joysick #1 address */

void timestamp(void) /* print current time */
{
  time_t start;
  struct tm now;

  time(&start);
  now=*localtime(&start);
  printf("%s%d:%s%d:%s%d\t",(now.tm_hour<10)?"0":"",now.tm_hour,(now.tm_min<10)?"0":"",now.tm_min,(now.tm_sec<10)?"0":"",now.tm_sec);
  fflush(stdout);
}

int main(int argc, char **argv)
{
  int status;
  int xa;
  char *gerr;
  int i;
  int calm = 0;         /* calmness status */
  int base = 0;         /* base resistance */
  BOOL cont = TRUE;     /* continue loop */
  BOOL perr = FALSE;    /* print error */
  BOOL stabstat = TRUE; /* stability status */
  BOOL maxresstat = TRUE;/* if(resistance < MAXRES) */
  long int p, oldp;  /* arit.average */
  unsigned long m;
  int *array;
  short c = 32;

  printf("odpok v0.3\n");
  printf("skin-resistance bio-feedback by nelo (nelo@message.sk)\n");

  if((argc != 1) && (argc != 4)) {
    printf("usage: odpok [<GAME_PORT_ADDR> <DELAY> <ITER>]\n");
    exit(1);
  } else if(argc == 4) {
    gaddr = strtol(argv[1], &gerr, 16);
    if(strlen(gerr) || gaddr <= 0) {
      printf("error: bad game port address\n");
      exit(1);
    }
    d_delay = atoi(argv[2]);
    if((d_delay < 0) || (d_delay > 100)) {
      printf("error: delay is out of range <0,100>\n");
      exit(1);
    }
    printf("%d\n",d_delay);
    d_iter = atoi(argv[3]);
    if((d_iter < 1) || (d_iter > 10000)) {
      printf("error: iter is out of range <1,10000>\n");
      exit(1);
    }
  }

  printf("(press space to calibrate, escape to exit)\n");
  printf("- game port base: %XH\n", gaddr);
  printf("- delay between readings: %d\n", d_delay);
  printf("- nuber of iterations: %d\n", d_iter);
  printf("time\t\tvalue\tcalmness\tstatus\n");

  if(!(array=(int *) malloc((d_iter)*sizeof(int)))) {
    printf("odpok: Not enough memory\n");
    exit(1);
  }

  /* let's find out values */
  oldp = 0;
  do {
      m = 0;
      stabstat = TRUE;
      maxresstat = TRUE;
      for(i = 0; i < d_iter + 1; i++) {
        xa = 0;
        status = 1;
        outp(gaddr,0);  /* start the resistance counting */
        while(inp(gaddr) & 1) {
          xa++;
          if (xa > MAXRES) {
            maxresstat = FALSE;
            break;
          }
        }
        if(!maxresstat) break;
        /* clear the capacitors */
        usleep(CAPACLR);
        /* get rid of noise */
        if(i && ((xa - MAXCALM > *(array + i - 1)) || (xa + MAXCALM < *(array + i - 1)))) {
          stabstat = FALSE;
        } else {
          *(array + i) = xa;
          m += xa;
        }
      }
      if(maxresstat && stabstat) {
        /* arit. average */
        p = m / d_iter;
        calm = 0;
        if((c != 32) && (perr != TRUE))
          calm = p - oldp;
        else calm = 0;
        oldp = p;
        perr = FALSE;
      } else {
        p = base;
        oldp = 0;
        calm = 0;
      }

      /* timestamp */
      if(!perr) {
        timestamp();
        printf("%5ld\t%4d\t\t", p - base, calm);
        if(!maxresstat)
          printf("not available\n");
        else if(!stabstat)
          printf("stabilizing ...\n");
        else {
          if(c == 32) printf("calibrated\n");
          else if(calm > 0) {
            if(calm > 15) printf("very fast ");
            else if(calm > 10) printf("fast ");
            else if(calm > 5) printf("mediately ");
            else if(calm > 2) printf("slowly ");
            else printf("very slowly ");
            printf("rising\n");
          } else if(calm < 0) {
            if(calm < -15) printf("very fast ");
            else if(calm < -10) printf("fast ");
            else if(calm < -5) printf("mediately ");
            else if(calm < -2) printf("slowly ");
            else printf("very slowly ");
            printf("falling\n");
          } else printf("calmness steady\n");
        }
      }

      if(!maxresstat || !stabstat) perr = TRUE;

      /* sleep */
      sleep(d_delay);

      /* check the keyboard */
      c = 0;
      if(kbhit()) {
        switch(c=getch()) {
          case 32 :
                    base = p;
                    oldp = p;
                    break;
          case 27 :
                    cont = FALSE;
                    break;
          default :
                    break;
        }
        fflush(stdin);
      }
  } while (cont);

  free(array);
  exit(0);
}

